import { Button, Form } from 'antd';
import React, { useState, useMemo } from 'react';
import {
  ArrowUpOutlined,
  ArrowDownOutlined,
  DeleteOutlined,
  PlusOutlined,
} from '@ant-design/icons';
import position from '../Flow/node/fields/position';

function FormContent(props) {
  const { list, onChange, onSelect, onTableColumnChange } = props;
  const [currentItem, setCurrentItem] = useState(null);

  const linkedList = useMemo(() => {
    const result = list
      ?.map((item) => {
        const linked = item.props.linked;
        return linked ? Object.values(linked).flat() : [];
      })
      .flat();
    console.log(result);
    return result;
  }, [list]);

  const handleDelete = (index) => {
    let _list = [...list];
    _list.splice(index, 1);
    onChange(_list);
  };
  const handleUp = (index) => {
    let _list = [...list];
    // 跟上一位置换位置
    const temp = _list[index - 1];
    _list[index - 1] = _list[index];
    _list[index] = temp;
    onChange(_list);
  };
  const handleDown = (index) => {
    let _list = [...list];
    const temp = _list[index + 1];
    _list[index + 1] = _list[index];
    _list[index] = temp;
    onChange(_list);
  };

  const handleFormItemClick = (id) => {
    setCurrentItem(id[0]);
    onSelect(id);
  };

  return (
    <div style={{ width: 500, height: 636, overflow: 'auto' }}>
      {list.map((item, index) => {
        const btns = (
          <>
            {index !== 0 && (
              <ArrowUpOutlined
                style={{ marginLeft: 5, cursor: 'pointer' }}
                onClick={() => handleUp(index)}
              />
            )}
            {index !== list.length - 1 && (
              <ArrowDownOutlined
                style={{ marginLeft: 5, cursor: 'pointer' }}
                onClick={() => handleDown(index)}
              />
            )}
            <DeleteOutlined
              style={{ marginLeft: 5, cursor: 'pointer' }}
              onClick={() => handleDelete(index)}
            />
          </>
        );

        return (
          <FormItem
            key={item.props?.id}
            islinked={
              linkedList?.findIndex((id) => id === item.props?.id) !== -1
            } //不等于-1表示是被单选框关联的组件需要置灰
            active={item.props?.id === currentItem}
            onClick={handleFormItemClick}
            onTableColumnChange={onTableColumnChange}
            item={item}
            btns={btns}
          />
        );
      })}
    </div>
  );
}

function FormItem(props) {
  const {
    item,
    btns,
    active,
    onClick,
    onTableColumnChange,
    islinked = false,
  } = props;
  const { label, placeholder, required } = item.props;

  // 子控件激活id
  const [selectColumnID, setSelectColumnID] = useState('');

  // 新增列时通过id定位
  const addTableColumn = (event) => {
    // 记录当前表格uuid
    onTableColumnChange(item.props.id);
  };

  // 修改表格内部的控件顺序
  const changeIndex = (index, operate) => {
    let newCol = [...item.columns];
    const prev = newCol[index - 1];
    const next = newCol[index + 1];
    switch (operate) {
      case 'up':
        newCol[index - 1] = newCol[index];
        newCol[index] = prev;
        break;
      case 'down':
        newCol[index + 1] = newCol[index];
        newCol[index] = next;
        break;
      case 'delete':
        newCol.splice(index, 1);
        break;
    }
    onTableColumnChange(item.props.id, newCol);
  };

  return (
    <div
      style={{
        backgroundColor: islinked ? '#e4dfdf' : '#fff',
        marginBottom: 20,
        padding: '4px 12px',
        border: islinked ? '1px solid #bebcbc' : '1px solid #666',
        borderLeft:
          active && !item.isTable ? '10px solid #1890FF' : '1px solid #666',
      }}
      onClick={(e) => {
        // 停止冒泡
        e.stopPropagation();
        if (item.isColumn === undefined) {
          // 触发正常的事件
          onClick([item.props.id]);
        } else {
          // 触发列中的click事件
          onClick();
        }
      }}
    >
      <div
        style={{
          fontSize: 16,
          color: '#000',
          fontWeight: 'bold',
          position: 'relative',
        }}
      >
        {required && <i style={{ color: 'red' }}>*</i>}
        {label}
        <div
          style={{
            position: 'absolute',
            right: 0,
            top: 0,
            padding: '5px 10px',
          }}
        >
          {btns}
        </div>
      </div>
      {item.isTable ? (
        <div style={{ padding: '10px 0 5px 0' }}>
          {item.columns.map((column, index) => {
            // column的按钮和外部的控件按钮不一样
            const colBtns = (
              <>
                {index !== 0 && (
                  <ArrowUpOutlined
                    style={{ marginLeft: 5, cursor: 'pointer' }}
                    onClick={() => {
                      changeIndex(index, 'up');
                    }}
                  />
                )}
                {index !== item.columns.length - 1 && (
                  <ArrowDownOutlined
                    style={{ marginLeft: 5, cursor: 'pointer' }}
                    onClick={() => {
                      changeIndex(index, 'down');
                    }}
                  />
                )}
                <DeleteOutlined
                  style={{ marginLeft: 5, cursor: 'pointer' }}
                  onClick={() => {
                    changeIndex(index, 'delete');
                  }}
                />
              </>
            );
            return (
              <FormItem
                key={column.props?.id}
                item={column}
                active={active && column.props?.id === selectColumnID}
                onClick={() => {
                  setSelectColumnID(column.props.id);
                  onClick([item.props.id, column.props.id]);
                }}
                btns={colBtns}
              />
            );
          })}
          <Button
            type="dashed"
            block
            onClick={addTableColumn}
            icon={<PlusOutlined />}
          >
            点击添加列
          </Button>
        </div>
      ) : (
        <div style={{ color: '#999', fontSize: 16 }}>{placeholder}</div>
      )}
    </div>
  );
}

export default FormContent;
